﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Google.Apis.Script.v1;
using Google.Apis.Script.v1.Data;
using Newtonsoft.Json.Linq;

namespace GSpreadCOM.Domain
{
	class AppsScriptService
	{
		private ScriptService service;
		private String scriptId;

		public AppsScriptService(ScriptService service, String scriptId)
		{
			this.service = service;
			this.scriptId = (String.IsNullOrEmpty(scriptId)) ? Config.GetAttribute("script", "id") : scriptId;
		}

		public dynamic call(string function, Range range, params Object[] parameters)
		{
			return call(function, new Object[] { range.Document.Id, range.Sheet.Name, range.FromRow, range.ToRow, range.FromColumn, range.ToColumn }.Concat(parameters).ToArray());
		}

		public dynamic call(string function, params Object[] parameters)
		{
			ExecutionRequest request = new ExecutionRequest();
			request.Function = function;
			request.Parameters = new List<object>(parameters);

			ScriptsResource.RunRequest runReq = service.Scripts.Run(request, scriptId);

			try
			{
				Operation op = runReq.Execute();

				if (op.Error == null)
				{
					if (op.Response.ContainsKey("result"))
					{
						return op.Response["result"];
					}
				}
				else
				{
					IDictionary<string, object> error = op.Error.Details[0];
					Console.WriteLine("Script error message: {0}", error["errorMessage"]);
					if (error.Keys.Contains("scriptStackTraceElements") && error["scriptStackTraceElements"] != null)
					{
						Console.WriteLine("Script error stacktrace:");
						JArray st = (JArray) error["scriptStackTraceElements"];
						foreach (var trace in st)
						{
							Console.WriteLine("\t{0}: {1}", trace["function"], trace["lineNumber"]);
						}
					}
				}
			}
			catch (Google.GoogleApiException e)
			{
				 Console.WriteLine("Error calling API:\n{0}", e);
			}

			return null;
		}
	}
}
