﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Runtime.InteropServices;
using Google.GData.Spreadsheets;

namespace GSpreadCOM.Domain
{
	sealed class Cell
	{
		public CellEntry Entry { get; private set; }

		public int Row { get; private set; }
		public int Column { get; private set; }
		public string ColumnLiteral { get; private set; }

		public Cell(string address, CellEntry entry = null)
		{
			Entry = entry;

			address = address.Replace("$", "");

			Regex re = new Regex(@"\d+");
			Match m = re.Match(address);

			int columnLength = address.Length;

			if (m.Success)
			{
				uint row;
				UInt32.TryParse(m.Value, out row);
				Row = (int) row;

				columnLength = m.Index;
			}
			else
			{
				Row = 0;
			}

			if (columnLength == 0)
			{
				Column = 0;
				ColumnLiteral = "";
			}
			else
			{
				ColumnLiteral = address.Substring(0, columnLength).ToUpper();
				Column = LiteralToNumber(ColumnLiteral);
			}
		}

		public Cell(int row, int column, CellEntry entry = null)
		{
			Entry = entry;

			Row = row;
			Column = column;
			ColumnLiteral = (Column > 0) ? NumberToLiteral(Column) : "";
		}

		public string GetAddressR1C1(bool rowAbsolute = true, bool columnAbsolute = true, Cell relativeTo = null)
		{
			int rowDelta = 0;
			int columnDelta = 0;

			if (!(rowAbsolute && columnAbsolute))
			{
				if (relativeTo == null) {
					relativeTo = new Cell(1, 1);
				}

				rowDelta = Row - relativeTo.Row;
				columnDelta = Column - relativeTo.Column;
			}

			return
				(Row == 0 ? "" : "R" + (rowAbsolute ? Row.ToString() : (rowDelta == 0 ? "" : String.Format("[{0}]", rowDelta)))) +
				(Column == 0 ? "" : "C" + (columnAbsolute ? Column.ToString() : (columnDelta == 0 ? "" : String.Format("[{0}]", columnDelta))));
		}

		public string GetAddressA1(bool rowAbsolute = true, bool columnAbsolute = true)
		{
			return
				(Column == 0 ? "" : (columnAbsolute ? "$" : "") + ColumnLiteral) +
				(Row == 0 ? "" : (rowAbsolute ? "$" : "") + Row.ToString());
		}

		private int LiteralToNumber(string literal)
		{
			string literalNum = literal.ToUpper();
			int number = 0;
			for (int i = literalNum.Length - 1; i >= 0; i--)
			{
				char letter = literalNum[i];
				int digit = letter - 64;
				number = number + digit * (int) Math.Pow(26, literalNum.Length - (i + 1));
			}

			return number;
		}

		private string NumberToLiteral(int number)
		{
			string literalNum = "";
			decimal num = number;
			while (num > 0)
			{
				decimal letterNum = (num - 1) % 26;
				literalNum = ((char) (letterNum + 65)) + literalNum;
				num = (num - (letterNum + 1)) / 26;
			}

			return literalNum;
		}
	}
}