﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Google.GData.Client;
using Google.GData.Spreadsheets;

namespace GSpreadCOM.Domain
{
	sealed class CellService
	{
		private Sheet sheet;
		private SpreadsheetsService spreadsheets;
		private CellQuery cellQuery;
		private CellFeed cellFeed;
		private Dictionary<String, Cell> cells;

		public CellService(Sheet sheet, SpreadsheetsService spreadsheets)
		{
			this.sheet = sheet;
			this.spreadsheets = spreadsheets;
			this.cells = new Dictionary<String, Cell>();
		}

		public Cell this[int RowIndex, int ColumnIndex]
		{
			get
			{
				Cell cell;
				String r1c1 = GetR1C1(RowIndex, ColumnIndex);
				if (cells.ContainsKey(r1c1))
				{
					cell = cells[r1c1];
				}
				else
				{
					CellEntry entry = new CellEntry((uint) RowIndex, (uint) ColumnIndex);
					entry.Id = new AtomId(String.Format("{0}/{1}", cellFeed.Self, GetR1C1(entry)));
					entry.Etag = "*";

					cell = new Cell(RowIndex, ColumnIndex, entry);
					cells.Add(r1c1, cell);
				}

				return cell;
			}
		}

		public void Load()
		{
			cells = new Dictionary<String, Cell>();
			cellQuery = new CellQuery(sheet.Worksheet.CellFeedLink);
			cellFeed = spreadsheets.Query(cellQuery);

			foreach (CellEntry entry in cellFeed.Entries)
			{
				cells.Add(GetR1C1(entry), new Cell((int) entry.Row, (int) entry.Column, entry));
			}
		}

		private String GetR1C1(CellEntry cell)
		{
			return GetR1C1((int) cell.Row, (int) cell.Column);
		}

		private String GetR1C1(int RowIndex, int ColumnIndex)
		{
			return String.Format("R{0}C{1}", RowIndex, ColumnIndex);
		}

		public void Save()
		{
			CellFeed batchRequest = new CellFeed(cellQuery.Uri, spreadsheets);
			foreach (Cell cell in cells.Values)
			{
				if (cell.Entry.Dirty)
				{
					cell.Entry.BatchData = new GDataBatchEntryData(GDataBatchOperationType.update);
					batchRequest.Entries.Add(cell.Entry);
				}
			}
			if (batchRequest.Entries.Count > 0)
			{
				spreadsheets.Batch(batchRequest, new Uri(cellFeed.Batch));
			}
		}
	}
}
