﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Globalization;

namespace GSpreadCOM.Domain
{
	static class ExcelColorPalette
	{
		public static ExcelColor[] Colors = {
			new ExcelColor("Black", "#000000"),
			new ExcelColor("White", "#FFFFFF"),
			new ExcelColor("Red", "#FF0000"),
			new ExcelColor("Green", "#00FF00"),
			new ExcelColor("Blue", "#0000FF"),
			new ExcelColor("Yellow", "#FFFF00"),
			new ExcelColor("Magenta", "#FF00FF"),
			new ExcelColor("Cyan", "#00FFFF"),
			new ExcelColor("[Color 9]", "#800000"),
			new ExcelColor("[Color 10]", "#008000"),
			new ExcelColor("[Color 11]", "#000080"),
			new ExcelColor("[Color 12]", "#808000"),
			new ExcelColor("[Color 13]", "#800080"),
			new ExcelColor("[Color 14]", "#008080"),
			new ExcelColor("[Color 15]", "#C0C0C0"),
			new ExcelColor("[Color 16]", "#808080"),
			new ExcelColor("[Color 17]", "#9999FF"),
			new ExcelColor("[Color 18]", "#993366"),
			new ExcelColor("[Color 19]", "#FFFFCC"),
			new ExcelColor("[Color 20]", "#CCFFFF"),
			new ExcelColor("[Color 21]", "#660066"),
			new ExcelColor("[Color 22]", "#FF8080"),
			new ExcelColor("[Color 23]", "#0066CC"),
			new ExcelColor("[Color 24]", "#CCCCFF"),
			new ExcelColor("[Color 25]", "#000080"),
			new ExcelColor("[Color 26]", "#FF00FF"),
			new ExcelColor("[Color 27]", "#FFFF00"),
			new ExcelColor("[Color 28]", "#00FFFF"),
			new ExcelColor("[Color 29]", "#800080"),
			new ExcelColor("[Color 30]", "#800000"),
			new ExcelColor("[Color 31]", "#008080"),
			new ExcelColor("[Color 32]", "#0000FF"),
			new ExcelColor("[Color 33]", "#00CCFF"),
			new ExcelColor("[Color 34]", "#CCFFFF"),
			new ExcelColor("[Color 35]", "#CCFFCC"),
			new ExcelColor("[Color 36]", "#FFFF99"),
			new ExcelColor("[Color 37]", "#99CCFF"),
			new ExcelColor("[Color 38]", "#FF99CC"),
			new ExcelColor("[Color 39]", "#CC99FF"),
			new ExcelColor("[Color 40]", "#FFCC99"),
			new ExcelColor("[Color 41]", "#3366FF"),
			new ExcelColor("[Color 42]", "#33CCCC"),
			new ExcelColor("[Color 43]", "#99CC00"),
			new ExcelColor("[Color 44]", "#FFCC00"),
			new ExcelColor("[Color 45]", "#FF9900"),
			new ExcelColor("[Color 46]", "#FF6600"),
			new ExcelColor("[Color 47]", "#666699"),
			new ExcelColor("[Color 48]", "#969696"),
			new ExcelColor("[Color 49]", "#003366"),
			new ExcelColor("[Color 50]", "#339966"),
			new ExcelColor("[Color 51]", "#003300"),
			new ExcelColor("[Color 52]", "#333300"),
			new ExcelColor("[Color 53]", "#993300"),
			new ExcelColor("[Color 54]", "#993366"),
			new ExcelColor("[Color 55]", "#333399"),
			new ExcelColor("[Color 56]", "#333333")
		};

		public static ExcelColor GetByName(string name) {
			string nameLower = name.ToLower();
			foreach (ExcelColor color in Colors)
			{
				if (color.Name.ToLower().Equals(nameLower))
				{
					return color;
				}
			}

			return null;
		}

		public static ExcelColor GetByIndex(int index)
		{
			if (1 <= index && index <= 56)
			{
				return Colors[index - 1];
			}
			return Colors[1];
		}

		public static int HtmlColorToInt(string htmlColor)
		{
			return ColorTranslator.ToWin32(ColorTranslator.FromHtml(htmlColor));
		}

		public static string IntToHtmlColor(int color)
		{
			return ColorTranslator.ToHtml(ColorTranslator.FromWin32(color));
		}

		public static int GetColorIndex(string hrmlColor) {
			int result = ExcelColor.FAULT_VALUE;
			Color color = ColorTranslator.FromHtml(hrmlColor);
			double currentProximity = Double.MaxValue;
			for (int i = 0; i < Colors.Length; i++)
			{
				ExcelColor xlColor = Colors[i];
				double proximity = ExcelColorPalette.CalculateProximity(color, xlColor.Value);
				if (proximity < currentProximity)
				{
					currentProximity = proximity;
					result = i + 1;
				}
			}

			return result;
		}

		public static int GetColorIndex(ExcelColor color)
		{
			return GetColorIndex(color.HtmlValue);
		}

		private static double CalculateProximity(Color c1, Color c2)
		{
			int r = c2.R - c1.R;
			int g = c2.G - c1.G;
			int b = c2.B - c1.B;

			return Math.Sqrt(r * r + g * g + b * b);
		}
	}
}