﻿using System.Collections.Generic;
using Google.GData.Spreadsheets;

namespace GSpreadCOM.Domain
{
	sealed class SheetsService
	{
		private SpreadsheetsService spreadsheets;
		private SpreadsheetEntry spreadsheet;

		public Document Document { get; private set; }
		public List<Sheet> Items { get; private set; }

		public SheetsService(Document document, SpreadsheetsService spreadsheets, SpreadsheetEntry spreadsheet)
		{
			this.spreadsheets = spreadsheets;
			this.spreadsheet = spreadsheet;

			Document = document;
			Items = new List<Sheet>();

			foreach (WorksheetEntry sheet in spreadsheet.Worksheets.Entries)
			{
				Items.Add(new Sheet(this, spreadsheets, sheet));
			}
		}

		public Sheet Add(string title, uint rows = 1000, uint cols = 26)
		{
			WorksheetEntry worksheet = new WorksheetEntry();
			worksheet.Title.Text = title;

			if (rows > 0)
			{
				worksheet.Rows = rows;
			}

			if (cols > 0)
			{
				worksheet.Cols = cols;
			}

			Sheet sheet = new Sheet(this, spreadsheets, spreadsheets.Insert(spreadsheet.Worksheets, worksheet));

			Items.Add(sheet);

			return sheet;
		}

		public void Delete(Sheet sheet)
		{
			if (Items.Contains(sheet))
			{
				sheet.Delete();
				Items.Remove(sheet);
			}
		}
	}
}
