﻿function copySpreadsheet(spreadsheetId, newName) {
  SpreadsheetApp.openById(spreadsheetId).copy(newName);
}

function exportSpreadsheetToPdf(spreadsheetId, pdfFilename) {
  var spreadsheetFile = DriveApp.getFileById(spreadsheetId);
  pdfFilename = pdfFilename || spreadsheetFile.getName() + ".pdf";
  DriveApp.getRootFolder().createFile(spreadsheetFile.getAs('application/pdf')).setName(pdfFilename);
}

function getSheet(spreadsheetId, sheetName) {
  var spreadsheet = SpreadsheetApp.openById(spreadsheetId);
  var sheet = spreadsheet.getSheetByName(sheetName);
  return sheet;
}

function checkAndFixColumnCount(sheet, count) {
  var lastColumnPosition = sheet.getMaxColumns();
  if (count > lastColumnPosition) {
    sheet.insertColumnsAfter(lastColumnPosition, count - lastColumnPosition);
  }
}

function checkAndFixRowCount(sheet, count) {
  var lastRowPosition = sheet.getMaxRows();
  if (count > lastRowPosition) {
    sheet.insertRowsAfter(lastRowPosition, count - lastRowPosition);
  }
}

function getRange(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  var sheet = getSheet(spreadsheetId, sheetName);
  checkAndFixRowCount(sheet, toRow);
  checkAndFixColumnCount(sheet, toColumn);

  return sheet.getRange(fromRow, fromColumn, toRow - fromRow + 1, toColumn - fromColumn + 1);
}

function checkEquality(table, faultValue) {
  if (typeof faultValue === "undefined") {
    faultValue = "varies";
  }

  var topLeft = table[0][0];
  for (var i in table) {
    for (var j in table[i]) {
      if (topLeft != table[i][j]) {
        return faultValue;
      }
    }
  }

  return topLeft;
}

function getColumnWidth(spreadsheetId, sheetName, fromPosition, toPosition) {
  var sheet = getSheet(spreadsheetId, sheetName);
  checkAndFixColumnCount(sheet, toPosition);

  var columnWidth = sheet.getColumnWidth(fromPosition);
  for (var i = fromPosition; i <= toPosition; i++) {
    var width = sheet.getColumnWidth(i);
    if (width != columnWidth) {
      return -1;
    }
  }
  return columnWidth;
}

function setColumnWidth(spreadsheetId, sheetName, fromPosition, toPosition, width) {
  var sheet = getSheet(spreadsheetId, sheetName);
  checkAndFixColumnCount(sheet, toPosition);
  
  for (var i = fromPosition; i <= toPosition; i++) {
    sheet.setColumnWidth(i, width);
  }
}

function getRowHeight(spreadsheetId, sheetName, fromPosition, toPosition) {
  var sheet = getSheet(spreadsheetId, sheetName);
  checkAndFixRowCount(sheet, toPosition);

  var rowHeight = sheet.getRowHeight(fromPosition);
  for (var i = fromPosition; i <= toPosition; i++) {
    var height = sheet.getRowHeight(i);
    if (height != rowHeight) {
      return -1;
    }
  }
  return rowHeight;
}

function setRowHeight(spreadsheetId, sheetName, fromPosition, toPosition, height) {
  var sheet = getSheet(spreadsheetId, sheetName);
  checkAndFixRowCount(sheet, toPosition);

  for (var i = fromPosition; i <= toPosition; i++) {
    sheet.setRowHeight(i, height);
  }
}

function setBorders(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn, top, left, bottom, right, vertical, horizontal, htmlColor, style) {
  var borderStyle = SpreadsheetApp.BorderStyle.SOLID;
  if (style == "xlDash") {
    borderStyle = SpreadsheetApp.BorderStyle.DASHED;
  } else if (style == "xlDot") {
    borderStyle = SpreadsheetApp.BorderStyle.DOTTED;
  }
  getRange.apply(this, arguments).setBorder(top, left, bottom, right, vertical, horizontal, htmlColor, borderStyle);
}

function getBackgroundColor(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  return checkEquality(getRange.apply(this, arguments).getBackgrounds());
}

function setBackgroundColor(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn, htmlColor) {
  getRange.apply(this, arguments).setBackground(htmlColor);
}

function getFontColor(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  return checkEquality(getRange.apply(this, arguments).getFontColors());
}

function setFontColor(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn, htmlColor) {
  getRange.apply(this, arguments).setFontColor(htmlColor);
}

function getFontWeight(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  return checkEquality(getRange.apply(this, arguments).getFontWeights());
}

function setFontWeight(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn, weight) {
  getRange.apply(this, arguments).setFontWeight(weight);
}

function getFontStyle(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  return checkEquality(getRange.apply(this, arguments).getFontStyles());
}

function setFontStyle(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn, style) {
  getRange.apply(this, arguments).setFontStyle(style);
}

function getFontLine(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  return checkEquality(getRange.apply(this, arguments).getFontLines());
}

function setFontLine(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn, line) {
  getRange.apply(this, arguments).setFontLine(line);
}

function getFontSize(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  return checkEquality(getRange.apply(this, arguments).getFontSizes());
}

function setFontSize(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn, size) {
  getRange.apply(this, arguments).setFontSize(size);
}

function getHorizontalAlignment(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  return checkEquality(getRange.apply(this, arguments).getHorizontalAlignments());
}

function setHorizontalAlignment(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn, alignment) {
  getRange.apply(this, arguments).setHorizontalAlignment(alignment);
}

function getVerticalAlignment(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  return checkEquality(getRange.apply(this, arguments).getVerticalAlignments());
}

function setVerticalAlignment(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn, alignment) {
  getRange.apply(this, arguments).setVerticalAlignment(alignment);
}

function getNumberFormat(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  return checkEquality(getRange.apply(this, arguments).getNumberFormats());
}

function setNumberFormat(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn, format) {
  getRange.apply(this, arguments).setNumberFormat(format);
}

function getWrap(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  return checkEquality(getRange.apply(this, arguments).getWraps());
}

function setWrap(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn, wrap) {
  getRange.apply(this, arguments).setWrap(wrap);
}

function getValue(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  var value = checkEquality(getRange.apply(this, arguments).getValues(), null);
  return value == null ? null : "" + value;
}

function setValue(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn, value) {
  getRange.apply(this, arguments).setValue(value);
}

function getDisplayValue(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  return checkEquality(getRange.apply(this, arguments).getDisplayValues(), null);
}

function getFormula(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  return checkEquality(getRange.apply(this, arguments).getFormulas());
}

function setFormula(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn, formula) {
  getRange.apply(this, arguments).setFormula(formula);
}

function getFormulaR1C1(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  return checkEquality(getRange.apply(this, arguments).getFormulasR1C1());
}

function setFormulaR1C1(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn, formula) {
  getRange.apply(this, arguments).setFormulaR1C1(formula);
}

function clearRange(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  getRange.apply(this, arguments).clear();
}

function clearRangeContent(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  getRange.apply(this, arguments).clearContent();
}

function clearRangeFormat(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  getRange.apply(this, arguments).clearFormat();
}

function mergeRange(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  getRange.apply(this, arguments).merge();
}

function mergeRangeAcross(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  getRange.apply(this, arguments).mergeAcross();
}

function unmergeRange(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  getRange.apply(this, arguments).breakApart();
}

function shiftRangeToRight(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  var sheet = getRange.apply(this, arguments).getSheet();
  sheet.insertColumnsBefore(fromColumn, toColumn - fromColumn + 1);
}

function shiftRangeDown(spreadsheetId, sheetName, fromRow, toRow, fromColumn, toColumn) {
  var sheet = getRange.apply(this, arguments).getSheet();
  sheet.insertRowsBefore(fromRow, toRow - fromRow + 1);
}