using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using Microsoft.Office.Core;
using Google.GData.Spreadsheets;
using Google.GData.Client;
using GSpreadCOM.Domain;

namespace GSpreadCOM
{
	[TypeLibType(TypeLibTypeFlags.FCanCreate | TypeLibTypeFlags.FDual | TypeLibTypeFlags.FDispatchable)]
	//[ClassInterface(ClassInterfaceType.AutoDual)]
	[ComSourceInterfaces("Microsoft.Office.Interop.Excel.WorkbookEvents_Event")]
	[Guid("5EFFC18B-7C5C-4374-BC2E-DDC0FB0B20BD")]
	[ComVisible(true)]
	public class Workbook : Microsoft.Office.Interop.Excel._Workbook, Microsoft.Office.Interop.Excel.Workbook, Microsoft.Office.Interop.Excel.WorkbookEvents_Event
	{
		internal Workbooks Workbooks { get; private set; }
		internal Document Document { get; private set; }
		internal AppsScriptService ScriptService { get; private set; }

		internal Workbook(Workbooks workbooks, Document document)
		{
			Workbooks = workbooks;
			Document = document;
			ScriptService = Document.Documents.Application.Script;
			Worksheets = new Worksheets(this);
		}

		#region properties

		/// <summary>
		/// Returns an object that represents the active sheet (the sheet on top) in the specified workbook. Returns Nothing if no sheet is active.
		/// </summary>
		[DispId(307)]
		public dynamic ActiveSheet
		{
			get
			{
				return (Worksheets as Worksheets).Active;
			}
		}

		/// <summary>
		/// Returns an Application object that represents the creator of the specified object. Read-only.
		/// </summary>
		[DispId(148)]
		public Microsoft.Office.Interop.Excel.Application Application
		{
			get { return Workbooks.Application; }
		}

		/// <summary>
		/// Returns the name of the object as a string. Read-only String.
		/// </summary>
		[DispId(289)]
		public string FullName
		{
			get { return Name; }
		}

		/// <summary>
		/// Returns a String value that represents the name of the object.
		/// </summary>
		[DispId(110)]
		public string Name
		{
			get { return Document.Title; }
		}

		/// <summary>
		/// Synonym for Workbook.Worksheets.
		/// </summary>
		[DispId(485)]
		public Microsoft.Office.Interop.Excel.Sheets Sheets
		{
			get { return Worksheets; }
		}

		/// <summary>
		/// Returns a Sheets collection that represents all the worksheets in the specified workbook. Read-only Sheets object.
		/// </summary>
		[DispId(494)]
		public Microsoft.Office.Interop.Excel.Sheets Worksheets { get; private set; }

		#endregion

		#region methods

		/// <summary>
		/// Activates the first window associated with the workbook.
		/// </summary>
		[DispId(304)]
		public void Activate()
		{
			Workbooks.Active = this;
		}

		/// <summary>
		/// Closes the object.
		/// </summary>
		/// <param name="SaveChanges">Boolean</param>
		/// <param name="Filename">if specified and SaveChanges is true, will copy the spreadsheet</param>
		/// <param name="RouteWorkbook">ignored</param>
		[DispId(277)]
		public void Close(object SaveChanges, object Filename, [Optional]object RouteWorkbook)
		{
			if (SaveChanges != Type.Missing && (bool) SaveChanges)
			{
				SaveAs(
					Filename, FileFormat: null, Password: null, WriteResPassword: null,
					ReadOnlyRecommended: false, CreateBackup: false, AccessMode: Microsoft.Office.Interop.Excel.XlSaveAsAccessMode.xlNoChange,
					ConflictResolution: null, AddToMru: false, TextCodepage: null, TextVisualLayout: null, Local: true
				);
			}

			Workbooks.Close(this);
		}

		/// <summary>
		/// The ExportAsFixedFormat method is used to publish a workbook to the PDF format.
		/// </summary>
		/// <param name="Type">XlFixedFormatType.xlTypePDF is only supported</param>
		/// <param name="Filename"></param>
		/// <param name="Quality">ignored</param>
		/// <param name="IncludeDocProperties">ignored</param>
		/// <param name="IgnorePrintAreas">ignored</param>
		/// <param name="From">ignored</param>
		/// <param name="To">ignored</param>
		/// <param name="OpenAfterPublish">ignored</param>
		/// <param name="FixedFormatExtClassPtr">ignored</param>
		[DispId(2493)]
		public void ExportAsFixedFormat(Microsoft.Office.Interop.Excel.XlFixedFormatType Type, object Filename, object Quality, object IncludeDocProperties, object IgnorePrintAreas, object From, object To, object OpenAfterPublish, object FixedFormatExtClassPtr)
		{
			if (Type == Microsoft.Office.Interop.Excel.XlFixedFormatType.xlTypePDF)
			{
				if (Filename == System.Type.Missing)
				{
					Filename = string.Empty;
				}
				ScriptService.call("exportSpreadsheetToPdf", Document.Id, Filename.ToString());
			}
		}

		/// <summary>
		/// Does nothing
		/// </summary>
		[DispId(283)]
		public void Save()
		{
		}

		/// <summary>
		/// Saves changes to the workbook in a different file.
		/// </summary>
		/// <param name="Filename"></param>
		/// <param name="FileFormat">ignored</param>
		/// <param name="Password">ignored</param>
		/// <param name="WriteResPassword">ignored</param>
		/// <param name="ReadOnlyRecommended">ignored</param>
		/// <param name="CreateBackup">ignored</param>
		/// <param name="AccessMode">ignored</param>
		/// <param name="ConflictResolution">ignored</param>
		/// <param name="AddToMru">ignored</param>
		/// <param name="TextCodepage">ignored</param>
		/// <param name="TextVisualLayout">ignored</param>
		/// <param name="Local">ignored</param>
		[DispId(1925)]
		public void SaveAs(object Filename, object FileFormat, object Password, object WriteResPassword, object ReadOnlyRecommended, object CreateBackup, Microsoft.Office.Interop.Excel.XlSaveAsAccessMode AccessMode, object ConflictResolution, object AddToMru, object TextCodepage, object TextVisualLayout, object Local)
		{
			ScriptService.call("copySpreadsheet", Document.Id, Filename.ToString());
		}

		[DispId(284)]
		public void _SaveAs(object Filename, object FileFormat, object Password, object WriteResPassword, object ReadOnlyRecommended, object CreateBackup, Microsoft.Office.Interop.Excel.XlSaveAsAccessMode AccessMode, object ConflictResolution, object AddToMru, object TextCodepage, object TextVisualLayout)
		{
		    SaveAs(Filename, FileFormat, Password, WriteResPassword, ReadOnlyRecommended, CreateBackup, AccessMode, ConflictResolution, AddToMru, TextCodepage, TextVisualLayout, null);
		}

		#endregion

		#region unused members

		[DispId(574)]
		public string Author
		{
			get { return Document.Author; }
			set { Document.Author = value; }
		}

		[DispId(575)]
		public string Comments
		{
			get { return Document.Summary; }
			set { Document.Summary = value; }
		}

		[DispId(-2147418112)]
		public string _CodeName { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(297)]
		public bool _ReadOnlyRecommended { get { throw new NotImplementedException(); } }

		[DispId(1441)]
		public bool AcceptLabelsInFormulas { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(183)]
		public Microsoft.Office.Interop.Excel.Chart ActiveChart { get { throw new NotImplementedException(); } }

		[DispId(1442)]
		public int AutoUpdateFrequency { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1443)]
		public bool AutoUpdateSaveChanges { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1176)]
		public dynamic BuiltinDocumentProperties { get { throw new NotImplementedException(); } }

		[DispId(1806)]
		public int CalculationVersion { get { throw new NotImplementedException(); } }

		[DispId(1444)]
		public int ChangeHistoryDuration { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(121)]
		public Microsoft.Office.Interop.Excel.Sheets Charts { get { throw new NotImplementedException(); } }

		[DispId(2528)]
		public bool CheckCompatibility { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1373)]
		public string CodeName { get { throw new NotImplementedException(); } }

		[DispId(1439)]
		public CommandBars CommandBars { get { throw new NotImplementedException(); } }

		[DispId(1175)]
		public Microsoft.Office.Interop.Excel.XlSaveConflictResolution ConflictResolution { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2513)]
		public Microsoft.Office.Interop.Excel.Connections Connections { get { throw new NotImplementedException(); } }

		[DispId(2536)]
		public bool ConnectionsDisabled { get { throw new NotImplementedException(); } }

		[DispId(1190)]
		public dynamic Container { get { throw new NotImplementedException(); } }

		[DispId(2512)]
		public MetaProperties ContentTypeProperties { get { throw new NotImplementedException(); } }

		[DispId(287)]
		public bool CreateBackup { get { throw new NotImplementedException(); } }

		[DispId(149)]
		public Microsoft.Office.Interop.Excel.XlCreator Creator { get { throw new NotImplementedException(); } }

		[DispId(1177)]
		public dynamic CustomDocumentProperties { get { throw new NotImplementedException(); } }

		[DispId(1456)]
		public Microsoft.Office.Interop.Excel.CustomViews CustomViews { get { throw new NotImplementedException(); } }

		[DispId(2530)]
		public CustomXMLParts CustomXMLParts { get { throw new NotImplementedException(); } }

		[DispId(403)]
		public bool Date1904 { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2527)]
		public dynamic DefaultPivotTableStyle { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2526)]
		public dynamic DefaultTableStyle { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(764)]
		public Microsoft.Office.Interop.Excel.Sheets DialogSheets { get { throw new NotImplementedException(); } }

		[DispId(404)]
		public Microsoft.Office.Interop.Excel.XlDisplayDrawingObjects DisplayDrawingObjects { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2276)]
		public bool DisplayInkComments { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2521)]
		public DocumentInspectors DocumentInspectors { get { throw new NotImplementedException(); } }

		[DispId(2274)]
		public DocumentLibraryVersions DocumentLibraryVersions { get { throw new NotImplementedException(); } }

		[DispId(2541)]
		public bool DoNotPromptForConvert { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2049)]
		public bool EnableAutoRecover { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2540)]
		public string EncryptionProvider { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1824)]
		public bool EnvelopeVisible { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(581)]
		public Microsoft.Office.Interop.Excel.Sheets Excel4IntlMacroSheets { get { throw new NotImplementedException(); } }

		[DispId(579)]
		public Microsoft.Office.Interop.Excel.Sheets Excel4MacroSheets { get { throw new NotImplementedException(); } }

		[DispId(2535)]
		public bool Excel8CompatibilityMode { get { throw new NotImplementedException(); } }

		[DispId(288)]
		public Microsoft.Office.Interop.Excel.XlFileFormat FileFormat { get { throw new NotImplementedException(); } }

		[DispId(2531)]
		public bool Final { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2542)]
		public bool ForceFullCalculation { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1927)]
		public string FullNameURLEncoded { get { throw new NotImplementedException(); } }

		[DispId(976)]
		public bool HasMailer { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(290)]
		public bool HasPassword { get { throw new NotImplementedException(); } }

		[DispId(950)]
		public bool HasRoutingSlip { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2529)]
		public bool HasVBProject { get { throw new NotImplementedException(); } }

		[DispId(1461)]
		public bool HighlightChangesOnScreen { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1823)]
		public HTMLProject HTMLProject { get { throw new NotImplementedException(); } }

		[DispId(2539)]
		public Microsoft.Office.Interop.Excel.IconSets IconSets { get { throw new NotImplementedException(); } }

		[DispId(2275)]
		public bool InactiveListBorderVisible { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1445)]
		public bool IsAddin { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1769)]
		public bool IsInplace { get { throw new NotImplementedException(); } }

		[DispId(1462)]
		public bool KeepChangeHistory { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(577)]
		public string Keywords { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1463)]
		public bool ListChangesOnNewSheet { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(979)]
		public Microsoft.Office.Interop.Excel.Mailer Mailer { get { throw new NotImplementedException(); } }

		[DispId(582)]
		public Microsoft.Office.Interop.Excel.Sheets Modules { get { throw new NotImplementedException(); } }

		[DispId(1169)]
		public bool MultiUserEditing { get { throw new NotImplementedException(); } }

		[DispId(442)]
		public Microsoft.Office.Interop.Excel.Names Names { get { throw new NotImplementedException(); } }

		[DispId(1178)]
		public string OnSave { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1031)]
		public string OnSheetActivate { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1081)]
		public string OnSheetDeactivate { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(150)]
		public dynamic Parent { get { return Workbooks; } }

		[DispId(429)]
		public string Password { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2060)]
		public string PasswordEncryptionAlgorithm { get { throw new NotImplementedException(); } }

		[DispId(2063)]
		public bool PasswordEncryptionFileProperties { get { throw new NotImplementedException(); } }

		[DispId(2061)]
		public int PasswordEncryptionKeyLength { get { throw new NotImplementedException(); } }

		[DispId(2059)]
		public string PasswordEncryptionProvider { get { throw new NotImplementedException(); } }

		[DispId(291)]
		public string Path { get { throw new NotImplementedException(); } }

		[DispId(2264)]
		public Permission Permission { get { throw new NotImplementedException(); } }

		[DispId(1447)]
		public bool PersonalViewListSettings { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1448)]
		public bool PersonalViewPrintSettings { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(405)]
		public bool PrecisionAsDisplayed { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(588)]
		public bool ProtectStructure { get { throw new NotImplementedException(); } }

		[DispId(295)]
		public bool ProtectWindows { get { throw new NotImplementedException(); } }

		[DispId(1819)]
		public Microsoft.Office.Interop.Excel.PublishObjects PublishObjects { get { throw new NotImplementedException(); } }

		[DispId(296)]
		public bool ReadOnly { get { return Document.IsReadOnly; } }

		[DispId(2005)]
		public bool ReadOnlyRecommended { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2050)]
		public bool RemovePersonalInformation { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2532)]
		public Microsoft.Office.Interop.Excel.Research Research { get { throw new NotImplementedException(); } }

		[DispId(1172)]
		public int RevisionNumber { get { throw new NotImplementedException(); } }

		[DispId(951)]
		public bool Routed { get { throw new NotImplementedException(); } }

		[DispId(949)]
		public Microsoft.Office.Interop.Excel.RoutingSlip RoutingSlip { get { throw new NotImplementedException(); } }

		[DispId(298)]
		public bool Saved { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(406)]
		public bool SaveLinkValues { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2519)]
		public ServerPolicy ServerPolicy { get { throw new NotImplementedException(); } }

		[DispId(2524)]
		public Microsoft.Office.Interop.Excel.ServerViewableItems ServerViewableItems { get { throw new NotImplementedException(); } }

		[DispId(2265)]
		public SharedWorkspace SharedWorkspace { get { throw new NotImplementedException(); } }

		[DispId(1171)]
		public bool ShowConflictHistory { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2538)]
		public bool ShowPivotChartActiveFields { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2046)]
		public bool ShowPivotTableFieldList { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2516)]
		public SignatureSet Signatures { get { throw new NotImplementedException(); } }

		[DispId(2273)]
		public SmartDocument SmartDocument { get { throw new NotImplementedException(); } }

		[DispId(2064)]
		public Microsoft.Office.Interop.Excel.SmartTagOptions SmartTagOptions { get { throw new NotImplementedException(); } }

		[DispId(493)]
		public Microsoft.Office.Interop.Excel.Styles Styles { get { throw new NotImplementedException(); } }

		[DispId(953)]
		public string Subject { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2266)]
		public Sync Sync { get { throw new NotImplementedException(); } }

		[DispId(2525)]
		public Microsoft.Office.Interop.Excel.TableStyles TableStyles { get { throw new NotImplementedException(); } }

		[DispId(1457)]
		public bool TemplateRemoveExtData { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2533)]
		public OfficeTheme Theme { get { throw new NotImplementedException(); } }

		[DispId(199)]
		public string Title { get { return Document.Title; } set { Document.Title = value; } }

		[DispId(864)]
		public Microsoft.Office.Interop.Excel.XlUpdateLinks UpdateLinks { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(411)]
		public bool UpdateRemoteReferences { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1210)]
		public bool UserControl { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1173)]
		public dynamic UserStatus { get { throw new NotImplementedException(); } }

		[DispId(1828)]
		public bool VBASigned { get { throw new NotImplementedException(); } }

		[DispId(1469)]
		public Microsoft.Vbe.Interop.VBProject VBProject { get { throw new NotImplementedException(); } }

		[DispId(1820)]
		public Microsoft.Office.Interop.Excel.WebOptions WebOptions { get { throw new NotImplementedException(); } }

		[DispId(430)]
		public Microsoft.Office.Interop.Excel.Windows Windows { get { throw new NotImplementedException(); } }

		[DispId(1128)]
		public string WritePassword { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(299)]
		public bool WriteReserved { get { throw new NotImplementedException(); } }

		[DispId(300)]
		public string WriteReservedBy { get { throw new NotImplementedException(); } }

		[DispId(2269)]
		public Microsoft.Office.Interop.Excel.XmlMaps XmlMaps { get { throw new NotImplementedException(); } }

		[DispId(2268)]
		public Microsoft.Office.Interop.Excel.XmlNamespaces XmlNamespaces { get { throw new NotImplementedException(); } }

		[DispId(905)]
		public void _PrintOut(object From, object To, object Copies, object Preview, object ActivePrinter, object PrintToFile, object Collate)
		{
			throw new NotImplementedException();
		}

		[DispId(282)]
		public void _Protect(object Password, object Structure, object Windows)
		{
			throw new NotImplementedException();
		}


		[DispId(1466)]
		public void AcceptAllChanges(object When, object Who, object Where)
		{
			throw new NotImplementedException();
		}


		[DispId(1476)]
		public void AddToFavorites()
		{
			throw new NotImplementedException();
		}


		[DispId(2534)]
		public void ApplyTheme(string Filename)
		{
			throw new NotImplementedException();
		}


		[DispId(2047)]
		public void BreakLink(string Name, Microsoft.Office.Interop.Excel.XlLinkType Type)
		{
			throw new NotImplementedException();
		}



		[DispId(2053)]
		public bool CanCheckIn()
		{
			throw new NotImplementedException();
		}


		[DispId(989)]
		public void ChangeFileAccess(Microsoft.Office.Interop.Excel.XlFileAccess Mode, object WritePassword, object Notify)
		{
			throw new NotImplementedException();
		}


		[DispId(802)]
		public void ChangeLink(string Name, string NewName, Microsoft.Office.Interop.Excel.XlLinkType Type = Microsoft.Office.Interop.Excel.XlLinkType.xlLinkTypeExcelLinks)
		{
			throw new NotImplementedException();
		}

		[DispId(2051)]
		public void CheckIn(object SaveChanges, object Comments, object MakePublic)
		{
			throw new NotImplementedException();
		}


		[DispId(2517)]
		public void CheckInWithVersion(object SaveChanges, object Comments, object MakePublic, object VersionType)
		{
			throw new NotImplementedException();
		}


		[DispId(397)]
		public void DeleteNumberFormat(string NumberFormat)
		{
			throw new NotImplementedException();
		}


		[DispId(2048)]
		public void Dummy16()
		{
			throw new NotImplementedException();
		}


		[DispId(2044)]
		public void Dummy17(int calcid)
		{
			throw new NotImplementedException();
		}


		[DispId(2537)]
		public void EnableConnections()
		{
			throw new NotImplementedException();
		}


		[DispId(2058)]
		public void EndReview()
		{
			throw new NotImplementedException();
		}


		[DispId(1168)]
		public bool ExclusiveAccess()
		{
			throw new NotImplementedException();
		}

		[DispId(1470)]
		public void FollowHyperlink(string Address, object SubAddress, object NewWindow, object AddHistory, object ExtraInfo, object Method, object HeaderInfo)
		{
			throw new NotImplementedException();
		}


		[DispId(973)]
		public void ForwardMailer()
		{
			throw new NotImplementedException();
		}


		[DispId(286)]
		public dynamic get_Colors(object Index)
		{
			throw new NotImplementedException();
		}


		[DispId(2522)]
		public WorkflowTasks GetWorkflowTasks()
		{
			throw new NotImplementedException();
		}


		[DispId(2523)]
		public WorkflowTemplates GetWorkflowTemplates()
		{
			throw new NotImplementedException();
		}


		[DispId(1458)]
		public void HighlightChangesOptions(object When, object Who, object Where)
		{
			throw new NotImplementedException();
		}


		[DispId(807)]
		public dynamic LinkInfo(string Name, Microsoft.Office.Interop.Excel.XlLinkInfo LinkInfo, object Type, object EditionRef)
		{
			throw new NotImplementedException();
		}


		[DispId(808)]
		public dynamic LinkSources(object Type)
		{
			throw new NotImplementedException();
		}


		[DispId(2520)]
		public void LockServerFile()
		{
			throw new NotImplementedException();
		}


		[DispId(1446)]
		public void MergeWorkbook(object Filename)
		{
			throw new NotImplementedException();
		}

		[DispId(280)]
		public Microsoft.Office.Interop.Excel.Window NewWindow()
		{
			throw new NotImplementedException();
		}


		[DispId(803)]
		public void OpenLinks(string Name, object ReadOnly, object Type)
		{
			throw new NotImplementedException();
		}


		[DispId(1449)]
		public Microsoft.Office.Interop.Excel.PivotCaches PivotCaches()
		{
			throw new NotImplementedException();
		}


		[DispId(684)]
		public void PivotTableWizard(object SourceType, object SourceData, object TableDestination, object TableName, object RowGrand, object ColumnGrand, object SaveData, object HasAutoFormat, object AutoPage, object Reserved, object BackgroundQuery, object OptimizeCache, object PageFieldOrder, object PageFieldWrapCount, object ReadData, object Connection)
		{
			throw new NotImplementedException();
		}


		[DispId(1166)]
		public void Post(object DestName)
		{
			throw new NotImplementedException();
		}

		[DispId(1772)]
		public void PrintOut(object From, object To, object Copies, object Preview, object ActivePrinter, object PrintToFile, object Collate, object PrToFileName)
		{
			throw new NotImplementedException();
		}


		[DispId(2361)]
		public void PrintOutEx(object From, object To, object Copies, object Preview, object ActivePrinter, object PrintToFile, object Collate, object PrToFileName, object IgnorePrintAreas)
		{
			throw new NotImplementedException();
		}


		[DispId(281)]
		public void PrintPreview(object EnableChanges)
		{
			throw new NotImplementedException();
		}

		[DispId(2029)]
		public void Protect(object Password, object Structure, object Windows)
		{
			throw new NotImplementedException();
		}


		[DispId(1450)]
		public void ProtectSharing(object Filename, object Password, object WriteResPassword, object ReadOnlyRecommended, object CreateBackup, object SharingPassword)
		{
			throw new NotImplementedException();
		}


		[DispId(2543)]
		public void ProtectSharingEx(object Filename, object Password, object WriteResPassword, object ReadOnlyRecommended, object CreateBackup, object SharingPassword, object FileFormat)
		{
			throw new NotImplementedException();
		}


		[DispId(1464)]
		public void PurgeChangeHistoryNow(int Days, object SharingPassword)
		{
			throw new NotImplementedException();
		}


		[DispId(2065)]
		public void RecheckSmartTags()
		{
			throw new NotImplementedException();
		}


		[DispId(1452)]
		public void RefreshAll()
		{
			throw new NotImplementedException();
		}


		[DispId(1467)]
		public void RejectAllChanges(object When, object Who, object Where)
		{
			throw new NotImplementedException();
		}


		[DispId(1821)]
		public void ReloadAs(MsoEncoding Encoding)
		{
			throw new NotImplementedException();
		}


		[DispId(2514)]
		public void RemoveDocumentInformation(Microsoft.Office.Interop.Excel.XlRemoveDocInfoType RemoveDocInfoType)
		{
			throw new NotImplementedException();
		}


		[DispId(1453)]
		public void RemoveUser(int Index)
		{
			throw new NotImplementedException();
		}


		[DispId(977)]
		public void Reply()
		{
			throw new NotImplementedException();
		}


		[DispId(978)]
		public void ReplyAll()
		{
			throw new NotImplementedException();
		}


		[DispId(2057)]
		public void ReplyWithChanges(object ShowMessage)
		{
			throw new NotImplementedException();
		}


		[DispId(1468)]
		public void ResetColors()
		{
			throw new NotImplementedException();
		}

		[DispId(946)]
		public void Route()
		{
			throw new NotImplementedException();
		}


		[DispId(634)]
		public void RunAutoMacros(Microsoft.Office.Interop.Excel.XlRunAutoMacro Which)
		{
			throw new NotImplementedException();
		}


		[DispId(2278)]
		public void SaveAsXMLData(string Filename, Microsoft.Office.Interop.Excel.XmlMap Map)
		{
			throw new NotImplementedException();
		}


		[DispId(175)]
		public void SaveCopyAs(object Filename)
		{
			throw new NotImplementedException();
		}


		[DispId(1826)]
		public void sblt(string s)
		{
			throw new NotImplementedException();
		}


		[DispId(2267)]
		public void SendFaxOverInternet(object Recipients, object Subject, object ShowMessage)
		{
			throw new NotImplementedException();
		}


		[DispId(2054)]
		public void SendForReview(object Recipients, object Subject, object ShowMessage, object IncludeAttachment)
		{
			throw new NotImplementedException();
		}


		[DispId(947)]
		public void SendMail(object Recipients, object Subject, object ReturnReceipt)
		{
			throw new NotImplementedException();
		}


		[DispId(980)]
		public void SendMailer(object FileFormat, Microsoft.Office.Interop.Excel.XlPriority Priority = Microsoft.Office.Interop.Excel.XlPriority.xlPriorityNormal)
		{
			throw new NotImplementedException();
		}


		[DispId(286)]
		public void set_Colors(object Index, object RHS)
		{
			throw new NotImplementedException();
		}


		[DispId(809)]
		public void SetLinkOnData(string Name, object Procedure)
		{
			throw new NotImplementedException();
		}


		[DispId(2062)]
		public void SetPasswordEncryptionOptions(object PasswordEncryptionProvider, object PasswordEncryptionAlgorithm, object PasswordEncryptionKeyLength, object PasswordEncryptionFileProperties)
		{
			throw new NotImplementedException();
		}


		[DispId(2279)]
		public void ToggleFormsDesign()
		{
			throw new NotImplementedException();
		}


		[DispId(285)]
		public void Unprotect(object Password)
		{
			throw new NotImplementedException();
		}


		[DispId(1455)]
		public void UnprotectSharing(object SharingPassword)
		{
			throw new NotImplementedException();
		}


		[DispId(995)]
		public void UpdateFromFile()
		{
			throw new NotImplementedException();
		}

		[DispId(804)]
		public void UpdateLink(object Name, object Type)
		{
			throw new NotImplementedException();
		}


		[DispId(1818)]
		public void WebPagePreview()
		{
			throw new NotImplementedException();
		}


		[DispId(2270)]
		public Microsoft.Office.Interop.Excel.XlXmlImportResult XmlImport(string Url, out Microsoft.Office.Interop.Excel.XmlMap ImportMap, object Overwrite, object Destination)
		{
			throw new NotImplementedException();
		}


		[DispId(2277)]
		public Microsoft.Office.Interop.Excel.XlXmlImportResult XmlImportXml(string Data, out Microsoft.Office.Interop.Excel.XmlMap ImportMap, object Overwrite, object Destination)
		{
			throw new NotImplementedException();
		}


		public virtual event Microsoft.Office.Interop.Excel.WorkbookEvents_ActivateEventHandler WorkbookEvents_Event_Activate;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_AddinInstallEventHandler AddinInstall;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_AddinUninstallEventHandler AddinUninstall;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_AfterXmlExportEventHandler AfterXmlExport;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_AfterXmlImportEventHandler AfterXmlImport;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_BeforeCloseEventHandler BeforeClose;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_BeforePrintEventHandler BeforePrint;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_BeforeSaveEventHandler BeforeSave;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_BeforeXmlExportEventHandler BeforeXmlExport;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_BeforeXmlImportEventHandler BeforeXmlImport;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_DeactivateEventHandler Deactivate;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_NewSheetEventHandler NewSheet;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_OpenEventHandler Open;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_PivotTableCloseConnectionEventHandler PivotTableCloseConnection;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_PivotTableOpenConnectionEventHandler PivotTableOpenConnection;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_RowsetCompleteEventHandler RowsetComplete;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetActivateEventHandler SheetActivate;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetBeforeDoubleClickEventHandler SheetBeforeDoubleClick;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetBeforeRightClickEventHandler SheetBeforeRightClick;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetCalculateEventHandler SheetCalculate;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetChangeEventHandler SheetChange;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetDeactivateEventHandler SheetDeactivate;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetFollowHyperlinkEventHandler SheetFollowHyperlink;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetPivotTableUpdateEventHandler SheetPivotTableUpdate;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetSelectionChangeEventHandler SheetSelectionChange;
		public virtual event Microsoft.Office.Interop.Excel.WorkbookEvents_SyncEventHandler WorkbookEvents_Event_Sync;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_WindowActivateEventHandler WindowActivate;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_WindowDeactivateEventHandler WindowDeactivate;
		public event Microsoft.Office.Interop.Excel.WorkbookEvents_WindowResizeEventHandler WindowResize;

		event Microsoft.Office.Interop.Excel.WorkbookEvents_SyncEventHandler Microsoft.Office.Interop.Excel.WorkbookEvents_Event.Sync
		{
			add { throw new NotImplementedException(); }
			remove { throw new NotImplementedException(); }
		}

		event Microsoft.Office.Interop.Excel.WorkbookEvents_ActivateEventHandler Microsoft.Office.Interop.Excel.WorkbookEvents_Event.Activate
		{
			add { throw new NotImplementedException(); }
			remove { throw new NotImplementedException(); }
		}

		#endregion

		#region excel version specific members
		/**/

		public int AccuracyVersion
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public Microsoft.Office.Interop.Excel.Slicer ActiveSlicer
		{
			get { throw new NotImplementedException(); }
		}

		public bool CaseSensitive
		{
			get { throw new NotImplementedException(); }
		}

		public bool ChartDataPointTrack
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic DefaultSlicerStyle
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic DefaultTimelineStyle
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public void Dummy26()
		{
			throw new NotImplementedException();
		}

		public void Dummy27()
		{
			throw new NotImplementedException();
		}

		/*public Microsoft.Office.Interop.Excel.Model Model
		{
			get { throw new NotImplementedException(); }
		}*/

		public dynamic PivotTables
		{
			get { throw new NotImplementedException(); }
		}

		public Microsoft.Office.Interop.Excel.SlicerCaches SlicerCaches
		{
			get { throw new NotImplementedException(); }
		}

		public bool UseWholeCellCriteria
		{
			get { throw new NotImplementedException(); }
		}

		public bool UseWildcards
		{
			get { throw new NotImplementedException(); }
		}


		public event Microsoft.Office.Interop.Excel.WorkbookEvents_AfterSaveEventHandler AfterSave;

		//public event Microsoft.Office.Interop.Excel.WorkbookEvents_ModelChangeEventHandler ModelChange;

		public event Microsoft.Office.Interop.Excel.WorkbookEvents_NewChartEventHandler NewChart;

		//public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetBeforeDeleteEventHandler SheetBeforeDelete;

		//public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetLensGalleryRenderCompleteEventHandler SheetLensGalleryRenderComplete;

		public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetPivotTableAfterValueChangeEventHandler SheetPivotTableAfterValueChange;

		public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetPivotTableBeforeAllocateChangesEventHandler SheetPivotTableBeforeAllocateChanges;

		public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetPivotTableBeforeCommitChangesEventHandler SheetPivotTableBeforeCommitChanges;

		public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetPivotTableBeforeDiscardChangesEventHandler SheetPivotTableBeforeDiscardChanges;

		public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetPivotTableChangeSyncEventHandler SheetPivotTableChangeSync;

		//public event Microsoft.Office.Interop.Excel.WorkbookEvents_SheetTableUpdateEventHandler SheetTableUpdate;

		/*//**/

		#endregion
	}
}
