﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using Google.GData.Spreadsheets;
using GSpreadCOM.Domain;

namespace GSpreadCOM
{
	[TypeLibType(TypeLibTypeFlags.FCanCreate | TypeLibTypeFlags.FDual | TypeLibTypeFlags.FDispatchable)]
	//[ClassInterface(ClassInterfaceType.AutoDual)]
	[ComSourceInterfaces("Microsoft.Office.Interop.Excel.DocEvents_Event")]
	[Guid("55B6B0C3-6D1B-4965-B093-48A8969205AB")]
	[ComVisible(true)]
	public class Worksheet : Microsoft.Office.Interop.Excel._Worksheet, Microsoft.Office.Interop.Excel.Worksheet, Microsoft.Office.Interop.Excel.DocEvents_Event
	{
		internal Worksheets Worksheets { get; set; }
		internal Document Document { get; private set; }
		internal Sheet Sheet { get; private set; }
		internal AppsScriptService ScriptService { get; private set; }
		internal Range ActiveCell { get; set; }

		protected Range cells;

		internal Worksheet(Worksheets sheets, Sheet sheet)
		{
			Worksheets = sheets;
			Sheet = sheet;
			Document = Sheet.Service.Document;
			ScriptService = Document.Documents.Application.Script;
			cells = new Range(this);
			ActiveCell = new Range(cells, 1, 1, 1, 1);
		}

		#region properties

		/// <summary>
		/// Returns an Application object that represents the creator of the specified object. Read-only.
		/// </summary>
		[DispId(148)]
		public Microsoft.Office.Interop.Excel.Application Application { get { return Worksheets.Workbook.Application; } }

		/// <summary>
		/// Returns a Range object that represents all the cells on the worksheet (not just the cells that are currently in use).
		/// </summary>
		[DispId(238)]
		public Microsoft.Office.Interop.Excel.Range Cells
		{
			get
			{
				return cells;
		    }
		}

		/// <summary>
		/// Returns a Range object that represents all the columns on the active worksheet.
		/// </summary>
		[DispId(241)]
		public Microsoft.Office.Interop.Excel.Range Columns
		{
			get
			{
				return Cells.Columns;
			}
		}

		/// <summary>
		/// Returns or sets a String value that represents the object name.
		/// </summary>
		[DispId(110)]
		public string Name
		{
			get { return Sheet.Name; }
			set { Sheet.Name = value; }
		}

		/// <summary>
		/// Returns a Range object that represents a cell or a range of cells.
		/// </summary>
		/// <param name="Cell1"></param>
		/// <param name="Cell2"></param>
		/// <returns></returns>
		[DispId(197)]
		public Microsoft.Office.Interop.Excel.Range get_Range(object Cell1, [Optional]object Cell2)
		{
		    return cells.get_Range(Cell1, Cell2);
		}

		/// <summary>
		/// Returns a Range object that represents all the rows on the specified worksheet. Read-only Range object.
		/// </summary>
		[DispId(258)]
		public Microsoft.Office.Interop.Excel.Range Rows
		{
			get
			{
				return Cells.Rows;
			}
		}

		#endregion

		#region methods

		/// <summary>
		/// Makes the current sheet the active sheet.
		/// </summary>
		[DispId(304)]
		public void Activate()
		{
			(Application as Application).ActiveSheet = this;
		}

		/// <summary>
		/// Deletes the object.
		/// </summary>
		[DispId(117)]
		public void Delete()
		{
			Worksheets.Delete(this);
		}

		#endregion

		#region unused members

		[DispId(-2147418112)]
		public string _CodeName { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(648)]
		public int _DisplayRightToLeft { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(793)]
		public Microsoft.Office.Interop.Excel.AutoFilter AutoFilter { get { throw new NotImplementedException(); } }

		[DispId(792)]
		public bool AutoFilterMode { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1069)]
		public Microsoft.Office.Interop.Excel.Range CircularReference { get { throw new NotImplementedException(); } }

		[DispId(1373)]
		public string CodeName { get { throw new NotImplementedException(); } }

		[DispId(575)]
		public Microsoft.Office.Interop.Excel.Comments Comments { get { throw new NotImplementedException(); } }

		[DispId(789)]
		public Microsoft.Office.Interop.Excel.XlConsolidationFunction ConsolidationFunction { get { throw new NotImplementedException(); } }

		[DispId(790)]
		public dynamic ConsolidationOptions { get { throw new NotImplementedException(); } }

		[DispId(791)]
		public dynamic ConsolidationSources { get { throw new NotImplementedException(); } }

		[DispId(149)]
		public Microsoft.Office.Interop.Excel.XlCreator Creator { get { throw new NotImplementedException(); } }

		[DispId(2030)]
		public Microsoft.Office.Interop.Excel.CustomProperties CustomProperties { get { throw new NotImplementedException(); } }

		[DispId(643)]
		public bool DisplayAutomaticPageBreaks { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1435)]
		public bool DisplayPageBreaks { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1774)]
		public bool DisplayRightToLeft { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1156)]
		public bool EnableAutoFilter { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1424)]
		public bool EnableCalculation { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(2511)]
		public bool EnableFormatConditionsCalculation { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1157)]
		public bool EnableOutlining { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1158)]
		public bool EnablePivotTable { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1425)]
		public Microsoft.Office.Interop.Excel.XlEnableSelection EnableSelection { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(800)]
		public bool FilterMode { get { throw new NotImplementedException(); } }

		[DispId(1418)]
		public Microsoft.Office.Interop.Excel.HPageBreaks HPageBreaks { get { throw new NotImplementedException(); } }

		[DispId(1393)]
		public Microsoft.Office.Interop.Excel.Hyperlinks Hyperlinks { get { throw new NotImplementedException(); } }

		[DispId(486)]
		public int Index { get { throw new NotImplementedException(); } }

		[DispId(2259)]
		public Microsoft.Office.Interop.Excel.ListObjects ListObjects { get { throw new NotImplementedException(); } }

		[DispId(2021)]
		public Microsoft.Office.Core.MsoEnvelope MailEnvelope { get { throw new NotImplementedException(); } }

		[DispId(442)]
		public Microsoft.Office.Interop.Excel.Names Names { get { throw new NotImplementedException(); } }

		[DispId(502)]
		public dynamic Next { get { throw new NotImplementedException(); } }

		[DispId(625)]
		public string OnCalculate { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(629)]
		public string OnData { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(628)]
		public string OnDoubleClick { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(627)]
		public string OnEntry { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1031)]
		public string OnSheetActivate { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1081)]
		public string OnSheetDeactivate { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(102)]
		public Microsoft.Office.Interop.Excel.Outline Outline { get { throw new NotImplementedException(); } }

		[DispId(998)]
		public Microsoft.Office.Interop.Excel.PageSetup PageSetup { get { throw new NotImplementedException(); } }

		[DispId(150)]
		public dynamic Parent { get { throw new NotImplementedException(); } }

		[DispId(503)]
		public dynamic Previous { get { throw new NotImplementedException(); } }

		[DispId(292)]
		public bool ProtectContents { get { throw new NotImplementedException(); } }

		[DispId(293)]
		public bool ProtectDrawingObjects { get { throw new NotImplementedException(); } }

		[DispId(176)]
		public Microsoft.Office.Interop.Excel.Protection Protection { get { throw new NotImplementedException(); } }

		[DispId(1159)]
		public bool ProtectionMode { get { throw new NotImplementedException(); } }

		[DispId(294)]
		public bool ProtectScenarios { get { throw new NotImplementedException(); } }

		[DispId(1434)]
		public Microsoft.Office.Interop.Excel.QueryTables QueryTables { get { throw new NotImplementedException(); } }

		[DispId(1816)]
		public Microsoft.Office.Core.Scripts Scripts { get { throw new NotImplementedException(); } }

		[DispId(1433)]
		public string ScrollArea { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1377)]
		public Microsoft.Office.Interop.Excel.Shapes Shapes { get { throw new NotImplementedException(); } }

		[DispId(2016)]
		public Microsoft.Office.Interop.Excel.SmartTags SmartTags { get { throw new NotImplementedException(); } }

		[DispId(880)]
		public Microsoft.Office.Interop.Excel.Sort Sort { get { throw new NotImplementedException(); } }

		[DispId(407)]
		public double StandardHeight { get { throw new NotImplementedException(); } }

		[DispId(408)]
		public double StandardWidth { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1041)]
		public Microsoft.Office.Interop.Excel.Tab Tab { get { throw new NotImplementedException(); } }

		[DispId(401)]
		public bool TransitionExpEval { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(402)]
		public bool TransitionFormEntry { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(108)]
		public Microsoft.Office.Interop.Excel.XlSheetType Type { get { throw new NotImplementedException(); } }

		[DispId(412)]
		public Microsoft.Office.Interop.Excel.Range UsedRange { get { throw new NotImplementedException(); } }

		[DispId(558)]
		public Microsoft.Office.Interop.Excel.XlSheetVisibility Visible { get { throw new NotImplementedException(); } set { throw new NotImplementedException(); } }

		[DispId(1419)]
		public Microsoft.Office.Interop.Excel.VPageBreaks VPageBreaks { get { throw new NotImplementedException(); } }

		[DispId(1817)]
		public void _CheckSpelling(object CustomDictionary, object IgnoreUppercase, object AlwaysSuggest, object SpellLang, object IgnoreFinalYaa, object SpellScript)
		{
			throw new NotImplementedException();
		}

		[DispId(-5)]
		public dynamic _Evaluate(object Name)
		{
			throw new NotImplementedException();
		}

		[DispId(1027)]
		public void _PasteSpecial(object Format, object Link, object DisplayAsIcon, object IconFileName, object IconIndex, object IconLabel)
		{
			throw new NotImplementedException();
		}

		[DispId(905)]
		public void _PrintOut(object From, object To, object Copies, object Preview, object ActivePrinter, object PrintToFile, object Collate)
		{
			throw new NotImplementedException();
		}

		[DispId(282)]
		public void _Protect(object Password, object DrawingObjects, object Contents, object Scenarios, object UserInterfaceOnly)
		{
			throw new NotImplementedException();
		}

		[DispId(284)]
		public void _SaveAs(string Filename, object FileFormat, object Password, object WriteResPassword, object ReadOnlyRecommended, object CreateBackup, object AddToMru, object TextCodepage, object TextVisualLayout)
		{
			throw new NotImplementedException();
		}

		[DispId(760)]
		public dynamic Arcs(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(557)]
		public dynamic Buttons(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(279)]
		public void Calculate()
		{
			throw new NotImplementedException();
		}

		[DispId(1060)]
		public dynamic ChartObjects(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(824)]
		public dynamic CheckBoxes(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(505)]
		public void CheckSpelling(object CustomDictionary, object IgnoreUppercase, object AlwaysSuggest, object SpellLang)
		{
			throw new NotImplementedException();
		}

		[DispId(1437)]
		public void CircleInvalid()
		{
			throw new NotImplementedException();
		}

		[DispId(970)]
		public void ClearArrows()
		{
			throw new NotImplementedException();
		}

		[DispId(1436)]
		public void ClearCircles()
		{
			throw new NotImplementedException();
		}

		[DispId(551)]
		public void Copy(object Before, object After)
		{
			throw new NotImplementedException();
		}

		[DispId(88)]
		public dynamic DrawingObjects(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(772)]
		public dynamic Drawings(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(836)]
		public dynamic DropDowns(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(1)]
		public dynamic Evaluate(object Name)
		{
			throw new NotImplementedException();
		}

		[DispId(2493)]
		public void ExportAsFixedFormat(Microsoft.Office.Interop.Excel.XlFixedFormatType Type, object Filename, object Quality, object IncludeDocProperties, object IgnorePrintAreas, object From, object To, object OpenAfterPublish, object FixedFormatExtClassPtr)
		{
			throw new NotImplementedException();
		}

		[DispId(834)]
		public dynamic GroupBoxes(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(1113)]
		public dynamic GroupObjects(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(841)]
		public dynamic Labels(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(767)]
		public dynamic Lines(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(832)]
		public dynamic ListBoxes(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(637)]
		public void Move(object Before, object After)
		{
			throw new NotImplementedException();
		}

		[DispId(799)]
		public dynamic OLEObjects(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(826)]
		public dynamic OptionButtons(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(801)]
		public dynamic Ovals(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(211)]
		public void Paste(object Destination, object Link)
		{
			throw new NotImplementedException();
		}

		[DispId(1928)]
		public void PasteSpecial(object Format, object Link, object DisplayAsIcon, object IconFileName, object IconIndex, object IconLabel, object NoHTMLFormatting)
		{
			throw new NotImplementedException();
		}

		[DispId(771)]
		public dynamic Pictures(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(690)]
		public dynamic PivotTables(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(684)]
		public Microsoft.Office.Interop.Excel.PivotTable PivotTableWizard(object SourceType, object SourceData, object TableDestination, object TableName, object RowGrand, object ColumnGrand, object SaveData, object HasAutoFormat, object AutoPage, object Reserved, object BackgroundQuery, object OptimizeCache, object PageFieldOrder, object PageFieldWrapCount, object ReadData, object Connection)
		{
			throw new NotImplementedException();
		}

		[DispId(1772)]
		public void PrintOut(object From, object To, object Copies, object Preview, object ActivePrinter, object PrintToFile, object Collate, object PrToFileName)
		{
			throw new NotImplementedException();
		}

		[DispId(2361)]
		public void PrintOutEx(object From, object To, object Copies, object Preview, object ActivePrinter, object PrintToFile, object Collate, object PrToFileName, object IgnorePrintAreas)
		{
			throw new NotImplementedException();
		}

		[DispId(281)]
		public void PrintPreview(object EnableChanges)
		{
			throw new NotImplementedException();
		}

		[DispId(2029)]
		public void Protect(object Password, object DrawingObjects, object Contents, object Scenarios, object UserInterfaceOnly, object AllowFormattingCells, object AllowFormattingColumns, object AllowFormattingRows, object AllowInsertingColumns, object AllowInsertingRows, object AllowInsertingHyperlinks, object AllowDeletingColumns, object AllowDeletingRows, object AllowSorting, object AllowFiltering, object AllowUsingPivotTables)
		{
			throw new NotImplementedException();
		}

		[DispId(774)]
		public dynamic Rectangles(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(1426)]
		public void ResetAllPageBreaks()
		{
			throw new NotImplementedException();
		}

		[DispId(1925)]
		public void SaveAs(string Filename, object FileFormat, object Password, object WriteResPassword, object ReadOnlyRecommended, object CreateBackup, object AddToMru, object TextCodepage, object TextVisualLayout, object Local)
		{
			throw new NotImplementedException();
		}

		[DispId(908)]
		public dynamic Scenarios(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(830)]
		public dynamic ScrollBars(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(235)]
		public void Select(object Replace)
		{
			throw new NotImplementedException();
		}

		[DispId(1188)]
		public void SetBackgroundPicture(string Filename)
		{
			throw new NotImplementedException();
		}

		[DispId(794)]
		public void ShowAllData()
		{
			throw new NotImplementedException();
		}

		[DispId(409)]
		public void ShowDataForm()
		{
			throw new NotImplementedException();
		}

		[DispId(838)]
		public dynamic Spinners(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(777)]
		public dynamic TextBoxes(object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(285)]
		public void Unprotect(object Password)
		{
			throw new NotImplementedException();
		}

		[DispId(2260)]
		public Microsoft.Office.Interop.Excel.Range XmlDataQuery(string XPath, object SelectionNamespaces, object Map)
		{
			throw new NotImplementedException();
		}

		[DispId(2263)]
		public Microsoft.Office.Interop.Excel.Range XmlMapQuery(string XPath, object SelectionNamespaces, object Map)
		{
			throw new NotImplementedException();
		}

		public event Microsoft.Office.Interop.Excel.DocEvents_BeforeDoubleClickEventHandler BeforeDoubleClick;
		public event Microsoft.Office.Interop.Excel.DocEvents_BeforeRightClickEventHandler BeforeRightClick;
		public event Microsoft.Office.Interop.Excel.DocEvents_ChangeEventHandler Change;
		public event Microsoft.Office.Interop.Excel.DocEvents_DeactivateEventHandler Deactivate;
		public event Microsoft.Office.Interop.Excel.DocEvents_FollowHyperlinkEventHandler FollowHyperlink;
		public event Microsoft.Office.Interop.Excel.DocEvents_PivotTableUpdateEventHandler PivotTableUpdate;
		public event Microsoft.Office.Interop.Excel.DocEvents_SelectionChangeEventHandler SelectionChange;

		event Microsoft.Office.Interop.Excel.DocEvents_CalculateEventHandler Microsoft.Office.Interop.Excel.DocEvents_Event.Calculate
		{
			add { throw new NotImplementedException(); }
			remove { throw new NotImplementedException(); }
		}

		event Microsoft.Office.Interop.Excel.DocEvents_ActivateEventHandler Microsoft.Office.Interop.Excel.DocEvents_Event.Activate
		{
			add { throw new NotImplementedException(); }
			remove { throw new NotImplementedException(); }
		}

		#endregion

		#region excel version specific members
		/**/

		public int PrintedCommentPages
		{
			get { throw new NotImplementedException(); }
		}


		//public event Microsoft.Office.Interop.Excel.DocEvents_BeforeDeleteEventHandler BeforeDelete;

		//public event Microsoft.Office.Interop.Excel.DocEvents_LensGalleryRenderCompleteEventHandler LensGalleryRenderComplete;

		public event Microsoft.Office.Interop.Excel.DocEvents_PivotTableAfterValueChangeEventHandler PivotTableAfterValueChange;

		public event Microsoft.Office.Interop.Excel.DocEvents_PivotTableBeforeAllocateChangesEventHandler PivotTableBeforeAllocateChanges;

		public event Microsoft.Office.Interop.Excel.DocEvents_PivotTableBeforeCommitChangesEventHandler PivotTableBeforeCommitChanges;

		public event Microsoft.Office.Interop.Excel.DocEvents_PivotTableBeforeDiscardChangesEventHandler PivotTableBeforeDiscardChanges;

		public event Microsoft.Office.Interop.Excel.DocEvents_PivotTableChangeSyncEventHandler PivotTableChangeSync;

		//public event Microsoft.Office.Interop.Excel.DocEvents_TableUpdateEventHandler TableUpdate;

		/*//**/
		#endregion
	}
}
