﻿using System;
using System.Drawing;
using Microsoft.Office.Interop.Excel;
using System.Text.RegularExpressions;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace GSpreadTest
{
	class Program
	{
		static void Main(string[] args)
		{
			/**/
			Type excelAppType = Type.GetTypeFromProgID("GSpreadCOM.Application");
			/*/
			Type excelAppType = Type.GetTypeFromProgID("Excel.Application");
			/**/

			var app = Activator.CreateInstance(excelAppType) as Application;
			Workbook doc = app.Workbooks["Example1"];
			Worksheet sheet = doc.Worksheets[1];
			sheet.Activate();

			// Data

			Range cells = app.Cells;
			(cells[2, 2] as Range).Value = "Product";
			for (int i = 1; i < 5; i++)
			{
				(cells[2, 2 + i] as Range).Value = "Quarter " + i;
			}
			(cells[2, 7] as Range).Value = "Year";

			(cells[4, 2] as Range).Value = "Apples";
			(cells[5, 2] as Range).Value = "Pears";
			(cells[6, 2] as Range).Value = "Bananas";
			(cells[7, 2] as Range).Value = "Oranges";
			(cells[8, 2] as Range).Value = "Totals";

			var random = new Random();
			for (int row = 4; row < 8; row++)
			{
				for (int col = 3; col < 7; col++)
				{
					(cells[row, col] as Range).Value = random.Next(100) * 100;
				}
			}

			cells.Range["C8:G8"].FormulaR1C1 = "=SUM(R[-4]C[0]:R[-1]C[0])";
			cells.Range["G4:G7"].Formula = "=SUM(C4:F4)";

			// Styling

			Range tableRange = cells.Range["B2:G8"];
			tableRange.Borders.Color = "white";
			tableRange.NumberFormat = "$0,000"; // currency number format

			cells.Range["B2:B3"].Merge();
			cells.Range["C2:C3"].Merge();
			cells.Range["D2:D3"].Merge();
			cells.Range["E2:E3"].Merge();
			cells.Range["F2:F3"].Merge();
			cells.Range["G2:G3"].Merge();

			Range headerRange = cells.Range["B2:G3"];
			Range footerRange = cells.Range["B8:G8"];

			// color #0066CC has index 23 in excel color palette
			headerRange.Interior.ColorIndex = 23;
			footerRange.Interior.Color = "#0066CC";

			// white color has index 2 in excel color palette
			headerRange.Font.ColorIndex = 2;
			footerRange.Font.Color = "white";

			headerRange.Font.Bold = true;
			footerRange.Font.Bold = true;

			footerRange.RowHeight = 30;

			int oddRowColor = ColorTranslator.ToWin32(Color.FromArgb(184, 204, 228));
			int evenRowColor = ColorTranslator.ToWin32(Color.FromArgb(219, 229, 241));

			cells.Range["B4:G4"].Interior.Color = oddRowColor;
			cells.Range["B5:G5"].Interior.Color = evenRowColor;
			cells.Range["B6:G6"].Interior.Color = oddRowColor;
			cells.Range["B7:G7"].Interior.Color = evenRowColor;
		}

		static void TestRange(Range range, string[] values)
		{
			Assert.AreEqual(values[0], range.Address[RowAbsolute:false, ColumnAbsolute:false]);
			Assert.AreEqual(values[1], range.Address[RowAbsolute:false, ColumnAbsolute:false, ReferenceStyle:XlReferenceStyle.xlR1C1]);
			Assert.AreEqual(values[2], range.Address[RowAbsolute:true, ColumnAbsolute:false]);
			Assert.AreEqual(values[3], range.Address[RowAbsolute:true, ColumnAbsolute:false, ReferenceStyle:XlReferenceStyle.xlR1C1]);
			Assert.AreEqual(values[4], range.Address[RowAbsolute:false, ColumnAbsolute:true]);
			Assert.AreEqual(values[5], range.Address[RowAbsolute:false, ColumnAbsolute:true, ReferenceStyle:XlReferenceStyle.xlR1C1]);
			Assert.AreEqual(values[6], range.Address[RowAbsolute:true, ColumnAbsolute:true]);
			Assert.AreEqual(values[7], range.Address[RowAbsolute:true, ColumnAbsolute:true, ReferenceStyle:XlReferenceStyle.xlR1C1]);
		}

		static void TestRanges()
		{
			Type excelAppType = Type.GetTypeFromProgID("GSpreadCOM.Application");
			var app = Activator.CreateInstance(excelAppType) as Application;
			Workbook doc = app.Workbooks["Test workbook"];
			Worksheet sheet = doc.Worksheets.Item[1];
			sheet.Activate();

			Range cells = app.Cells;

			Range rangeBD = cells.Range["B:D"];
			Range range24 = cells.Range["2:4"];
			Range rangeB2D4 = cells.Range["B2:D4"];
			Range rangeBDRows = cells.Range["B:D"].Rows;
			Range range24Rows = cells.Range["2:4"].Rows;
			Range rangeB2D4Rows = cells.Range["B2:D4"].Rows;
			Range rangeBDColumns = cells.Range["B:D"].Columns;
			Range range24Columns = cells.Range["2:4"].Columns;
			Range rangeB2D4Columns = cells.Range["B2:D4"].Columns;
			Range rangeBDRows3 = cells.Range["B:D"].Rows[3];
			Range range24Rows3 = cells.Range["2:4"].Rows[3];
			Range rangeB2D4Rows3 = cells.Range["B2:D4"].Rows[3];
			Range rangeBDColumns3 = cells.Range["B:D"].Columns[3];
			Range range24Columns3 = cells.Range["2:4"].Columns[3];
			Range rangeB2D4Columns3 = cells.Range["B2:D4"].Columns[3];
			Range rangeBDRows3Columns3 = (cells.Range["B:D"].Rows[3] as Range).Columns[3];
			Range range24Rows3Columns3 = (cells.Range["2:4"].Rows[3] as Range).Columns[3];
			Range rangeB2D4Rows3Columns3 = (cells.Range["B2:D4"].Rows[3] as Range).Columns[3];

			TestRange(cells, new string[] { "1:1048576", "R:R[1048575]", "$1:$1048576", "R1:R1048576", "1:1048576", "R:R[1048575]", "$1:$1048576", "R1:R1048576" });
			TestRange(cells.Rows, new string[] { "1:1048576", "R:R[1048575]", "$1:$1048576", "R1:R1048576", "1:1048576", "R:R[1048575]", "$1:$1048576", "R1:R1048576" });
			TestRange(cells.Columns, new string[] { "1:1048576", "R:R[1048575]", "$1:$1048576", "R1:R1048576", "1:1048576", "R:R[1048575]", "$1:$1048576", "R1:R1048576" });

			TestRange(rangeBD, new string[] { "B:D", "C[1]:C[3]", "B:D", "C[1]:C[3]", "$B:$D", "C2:C4", "$B:$D", "C2:C4" });
			TestRange(range24, new string[] { "2:4", "R[1]:R[3]", "$2:$4", "R2:R4", "2:4", "R[1]:R[3]", "$2:$4", "R2:R4" });
			TestRange(rangeB2D4, new string[] { "B2:D4", "R[1]C[1]:R[3]C[3]", "B$2:D$4", "R2C[1]:R4C[3]", "$B2:$D4", "R[1]C2:R[3]C4", "$B$2:$D$4", "R2C2:R4C4" });

			TestRange(rangeBDRows, new string[] { "B:D", "C[1]:C[3]", "B:D", "C[1]:C[3]", "$B:$D", "C2:C4", "$B:$D", "C2:C4" });
			TestRange(range24Rows, new string[] { "2:4", "R[1]:R[3]", "$2:$4", "R2:R4", "2:4", "R[1]:R[3]", "$2:$4", "R2:R4" });
			TestRange(rangeB2D4Rows, new string[] { "B2:D4", "R[1]C[1]:R[3]C[3]", "B$2:D$4", "R2C[1]:R4C[3]", "$B2:$D4", "R[1]C2:R[3]C4", "$B$2:$D$4", "R2C2:R4C4" });

			TestRange(rangeBDColumns, new string[] { "B:D", "C[1]:C[3]", "B:D", "C[1]:C[3]", "$B:$D", "C2:C4", "$B:$D", "C2:C4" });
			TestRange(range24Columns, new string[] { "2:4", "R[1]:R[3]", "$2:$4", "R2:R4", "2:4", "R[1]:R[3]", "$2:$4", "R2:R4" });
			TestRange(rangeB2D4Columns, new string[] { "B2:D4", "R[1]C[1]:R[3]C[3]", "B$2:D$4", "R2C[1]:R4C[3]", "$B2:$D4", "R[1]C2:R[3]C4", "$B$2:$D$4", "R2C2:R4C4" });

			TestRange(rangeBDRows3, new string[] { "B3:D3", "R[2]C[1]:R[2]C[3]", "B$3:D$3", "R3C[1]:R3C[3]", "$B3:$D3", "R[2]C2:R[2]C4", "$B$3:$D$3", "R3C2:R3C4" });
			TestRange(range24Rows3, new string[] { "4:4", "R[3]", "$4:$4", "R4", "4:4", "R[3]", "$4:$4", "R4" });
			TestRange(rangeB2D4Rows3, new string[] { "B4:D4", "R[3]C[1]:R[3]C[3]", "B$4:D$4", "R4C[1]:R4C[3]", "$B4:$D4", "R[3]C2:R[3]C4", "$B$4:$D$4", "R4C2:R4C4" });

			TestRange(rangeBDColumns3, new string[] { "D:D", "C[3]", "D:D", "C[3]", "$D:$D", "C4", "$D:$D", "C4" });
			TestRange(range24Columns3, new string[] { "C2:C4", "R[1]C[2]:R[3]C[2]", "C$2:C$4", "R2C[2]:R4C[2]", "$C2:$C4", "R[1]C3:R[3]C3", "$C$2:$C$4", "R2C3:R4C3" });
			TestRange(rangeB2D4Columns3, new string[] { "D2:D4", "R[1]C[3]:R[3]C[3]", "D$2:D$4", "R2C[3]:R4C[3]", "$D2:$D4", "R[1]C4:R[3]C4", "$D$2:$D$4", "R2C4:R4C4" });

			TestRange(rangeBDRows3Columns3, new string[] { "D3", "R[2]C[3]", "D$3", "R3C[3]", "$D3", "R[2]C4", "$D$3", "R3C4" });
			TestRange(range24Rows3Columns3, new string[] { "C4", "R[3]C[2]", "C$4", "R4C[2]", "$C4", "R[3]C3", "$C$4", "R4C3" });
			TestRange(rangeB2D4Rows3Columns3, new string[] { "D4", "R[3]C[3]", "D$4", "R4C[3]", "$D4", "R[3]C4", "$D$4", "R4C4" });


			Range cellR2C2 = cells[2, 2];
			Range cellR3C3 = cells[3, 3];
			Range cellR5C5 = cells[5, 5];
			Range cellR10C10 = cells[10, 10];
			Range cellR30C30 = cells[30, 30];

			try
			{
				Range invalid = app.Range["B2:D4"].Range["5:6"];
				Assert.Fail();
			}
			catch (InvalidOperationException e) { }

			Assert.AreEqual("$8:$9", app.Range["A4:D4"].Range["5:6"].Address);
			Assert.AreEqual("$H:$I", app.Range["C1:D4"].Range["F:G"].Address);

			Assert.AreEqual("R4C[-26]:R6C[-24]", app.Range[cellR2C2, cellR10C10].Range["$C$3", "E5"].Address[ReferenceStyle:XlReferenceStyle.xlR1C1, ColumnAbsolute:false, RelativeTo:cellR30C30]);
			Assert.AreEqual("$D$4:$F$6", app.Range[cellR2C2, cellR10C10].Range["$C$3", "E5"].Address);

			Assert.AreEqual("$D$4", (app.Range["C:E"][11] as Range).Address);
			Assert.AreEqual("$C$8", ((app.Range["B2:D4"][20] as Range).Item[1, 1] as Range).Address);
			Assert.AreEqual("$F$3", (app.Range["2:4"][16390] as Range).Address);

			Assert.AreEqual("$2:$3", app.Range["B2:D3"].EntireRow.Address);
			Assert.AreEqual("$B:$D", app.Range["B2:D3"].EntireColumn.Address);

			Range E5 = app.Range["E5"];
			dynamic valueOfE5 = E5.Value;
			E5.Value = "test1";
			Assert.AreEqual(E5.Value, ((app.Range[cellR2C2, cellR10C10].Range["$C$3", "E5"].Rows[2] as Range).Cells.Columns[2] as Range).Cells.Cells.Value);
			E5.Value = valueOfE5;

			Range F5 = app.Range["F5"];
			dynamic valueOfF5 = F5.Value;
			F5.Value = "test2";
			Assert.AreEqual(F5.Value, ((app.Range["B2", "J10"].Range[cellR3C3, cellR5C5].Rows[2] as Range).Columns[3] as Range).Cells.Cells.Value);
			F5.Value = valueOfF5;
		}
	}
}